unit WSFilter;
(* QUESTA <UNIT> NON E' ANCORA DEFINITA *)
(* CREA SOLO FILTRI PASSABASSO/PASSALTO *)
(* Supporta i seguenti filtri:
   * Butterworth
   * Chebyshev  (inband ripple)
   * Inverse Chebyshev (stopband attenuation)
*)

{
 This unit contains the data and procedure
 used to build/simulate numerical filters

 L.C. 3-2-1994 Ver 0.2
      25-6-1994 Ver 0.3, updated with <StoreFilterParam>,
                                      <GetFilterDlg>.

 Used by WSwnd.PAS, WSDLL.PAS

AP1: gestisce solo ingressi,uscite e coefficienti reali
AP2: la procedura <Coeff> e' precisa ma ha complessita' esponenziale! 

}


interface
uses WinTypes, WinProcs, WObjects, Strings, WsObj;

type pArray=^RealArray;
(*            M       
             Sum(B(k) * z^-k)
             k=0  
      H(z)=--------------------
                N
           1 - Sum(A(k) * z^-k)
               k=1
*)
type   pfilter=^tfilter;
       tfilter=object
               constructor Init(_n,_m:integer);
               destructor Done;
               procedure execfilter(var input,output:tiobuf);
               procedure reset;
               procedure FreqResponse(var response:tiobuf);
{              procedure getfilter();
               procedure putfilter();
}
               private
               pa:pArray;
               pb:pArray;
               n,m:integer;
               pXz,pYz:pArray;
               end;
        pBuiltIn=^tBuiltIn;
        tBuiltIn=object(tfilter)
               filtype,filtname:integer;
               ft,ripple:real;
               order:integer;
               end;

const zLowPass=0; (* Used as filtype in <NewBuiltIn> *)
      zHighPass=1;

      zButter=0;  (* Used as filtname in <NewBuiltIn> *)
      zCheb=1;
      zInvCheb=2;

procedure SetUpFilterDialog(HWindow:HWND;id:integer;filterp:pBuiltIn);
procedure FilterDialogOk(HWindow:HWND;id:integer;var newp:pBuiltIn); (* don't use it, old procedure *)
function NewBuiltIn(var filterp:pBuiltIn;n:integer;fc,ripple:real;
          _filtype,_filtname:integer):integer; (* don't use it, old procedure *)
function StoreFilterParam(PS:pSystem;var filterp:pBuiltIn;HWindow:HWnd;var st:string;ErrString:PChar):boolean;
procedure GetFilterDlg(HWindow:HWND;id:integer;var st:string);

implementation

(*---- TFilter ---*)

constructor TFilter.Init(_n,_m:integer);
(* creates a filter with unknown coefficients values *)
begin
  n:=_n;m:=_m;
  if n>0
  then begin
       GetMem(pa,(n+1)*sizeof(real));   (*+1:per semplicita'*)
       GetMem(pYz,n*sizeof(real));
       end;
  if m>=0
  then begin
       GetMem(pb,(m+1)*sizeof(real));
       if m <> 0
       then GetMem(pXz,m*sizeof(real));
       end;
  reset;
end;

destructor TFilter.Done;
begin
  if n>0
  then begin
       FreeMem(pa,(n+1)*sizeof(real));
       FreeMem(pYz,n*sizeof(real));
       end;
  if m>=0
  then begin
       FreeMem(pb,(m+1)*sizeof(real));
       if m <> 0
       then FreeMem(pXz,m*sizeof(real));
       end;
end;

procedure TFilter.execfilter(var input,output:tiobuf);
var k:integer;
    i:longint;
    sum:real;
begin
for i:=0 to input.size-1
do begin
   sum:=0;
(* VERIFICARE ALGORITMO *)
   if i<m
   then begin
        for k:=0 to i
        do sum:=sum+pb^[k]*pArray(input.buf)^[i-k];
        for k:=i+1 to m
        do sum:=sum+pb^[k]*pXz^[i-k+m];
        end
   else for k:=0 to m
        do sum:=sum+pb^[k]*pArray(input.buf)^[i-k];
   if i<n
   then begin
        for k:=1 to i
        do sum:=sum+pa^[k]*pArray(output.buf)^[i-k];
        for k:=i+1 to n
        do sum:=sum+pa^[k]*pYz^[i-k+n];
        end
   else for k:=1 to n
        do sum:=sum+pa^[k]*pArray(output.buf)^[i-k];
   pArray(output.buf)^[i]:=sum;
   end;
if input.size<m
then begin
     for k:=m-input.size-1 downto 0
     do pXz^[k]:=pXz^[k+input.size];
     for k:=m-input.size to m-1
     do pXz^[k]:=pArray(input.buf)^[k+input.size-m];
     end
else for k:=0 to m-1
     do pXz^[k]:=pArray(input.buf)^[k+input.size-m];
if input.size<n
then begin
     for k:=n-input.size-1 downto 0
     do pYz^[k]:=pYz^[k+input.size];
     for k:=n-input.size to n-1
     do pYz^[k]:=pArray(output.buf)^[k+input.size-n];
     end
else for k:=0 to n-1
     do pYz^[k]:=pArray(output.buf)^[k+input.size-n];
end;

procedure TFilter.reset;
var i:integer;
begin
  for i:=0 to n-1
  do pYz^[i]:=0;
  for i:=0 to m-1
  do pXz^[i]:=0;
end;

procedure TFilter.FreqResponse(var response:tiobuf);
var k:integer;
    i:longint;
    cosw,sinw,delta,sinkw,coskw,temp:real;
    num,den,sumreal,sumim:real;
begin
delta:=Pi/response.size;
for i:=0 to response.size-1
do begin
   cosw:=cos(i*delta);coskw:=1;
   sinw:=sin(i*delta);sinkw:=0;
   sumreal:=0;sumim:=0;
   for k:=0 to M
   do begin
      sumreal:=sumreal+pb^[k]*coskw;
      sumim:=sumim+pb^[k]*sinkw;
      temp:=cosw*coskw-sinw*sinkw;
      sinkw:=cosw*sinkw+sinw*coskw;
      coskw:=temp;
      end;
   num:=sqr(sumim)+sqr(sumreal);

   coskw:=cosw;sinkw:=sinw;
   sumreal:=1;sumim:=0;
   for k:=1 to N
   do begin
      sumreal:=sumreal-pa^[k]*coskw;
      sumim:=sumim+pa^[k]*sinkw;
      temp:=cosw*coskw-sinw*sinkw;
      sinkw:=cosw*sinkw+sinw*coskw;
      coskw:=temp;
      end;
   den:=sqr(sumim)+sqr(sumreal);

   pArray(response.buf)^[i]:=sqrt(num/den);
   end;
end;

(*---- internal ----*)

(* Progetto filtri numerici passabasso *)


const MaxOrder=1024;
type cplx=record
            r,i:real;
          end;
type CVett=array[1..MaxOrder]of cplx;
     Tindex=array[1..MaxOrder]of integer;

var P,Z:CVett;

procedure S2Z(var t:cplx);
(* bilinear, convert s-->z *)
var d:real;
begin
  d:=sqr(2-t.r)+sqr(t.i);
  t.r:=(4-sqr(t.r)-sqr(t.i))/d;
  t.i:=4*t.i/d;
end;

procedure CalcolaPoli(aa,bb:real;n:integer);
var al:real;
    i:integer;
begin
(* calcola i poli sulla circonferenza *)
for I:=1 to n
do begin
   AL:=(I/n-1/2/n)*PI;
   P[I].r:=-aa*sin(AL);
   P[I].i:=bb*cos(AL)
   end;
(* applica la trasformazione ai poli *)
for i:=1 to n
do S2Z(P[i]);
(* dopo la trasformazione bilineare ci sono <n> zeri in -1 ! *)
for i:=1 to n
do begin
    Z[i].r:=-1;Z[i].i:=0;
   end;
end;

function next(var a:Tindex;k,n:integer):boolean;
var i,l:integer;
begin
   next:=true;
   if a[k]=n
   then begin
        l:=k;
        while (l<>0)and(a[l]=n+l-k)
        do dec(l);
        if (l<>0)
        then begin
             inc(a[l]);
             for i:=l+1 to k
             do a[i]:=a[i-1]+1;
             end
        else next:=false;
        end
   else inc(a[k]);
end;

procedure init(var a:Tindex;k:integer);
var i:integer;
begin
for i:=1 to k
do a[i]:=i;
end;

procedure prod(var a,b:cplx);
(* complex product a*b->a *)
var c:cplx;
begin
  c.r:=a.r*b.r-a.i*b.i;
  c.i:=a.i*b.r+a.r*b.i;
  a:=c;
end;

procedure Coeff(var a,b:RealArray;n:integer);
(* AP2: questa procedura e' precisa ma ha complessita' esponenziale! *)
var i,k:integer;
    index:Tindex;
    sign:real;
    sum,norm:real;
    fatt:cplx;
begin
  sign:=1;
  for k:=1 to n
  do begin
     init(index,k);
     sum:=0;
     repeat
       fatt:=P[index[1]];
       for i:=2 to k
       do prod(fatt,P[index[i]]);
       sum:=sum + fatt.r; (* fatt.i = 0 !*)
     until not next(index,k,n);
     a[k]:=sum * sign;
     sign:=-sign;
     end;

(* dopo la trasformazione bilineare ci sono <n> zeri in -1 ! *)
  sign:=-1;
  b[0]:=1;
  for k:=1 to n
  do begin
     init(index,k);
     sum:=0;
     repeat
       fatt:=Z[index[1]];
       for i:=2 to k
       do prod(fatt,Z[index[i]]);
       sum:=sum + fatt.r; (* fatt.i = 0 !*)
     until not next(index,k,n);
     b[k]:=sum * sign;
     sign:=-sign;
     end;
end;

procedure AdjustGain(z:real;var a,b:RealArray;n:integer);
var zk:real;
    sum,norm:real;
    k:integer;
begin
(* normalizza <b> *)
  sum:=1;
  zk:=z;
  for k:=1 to n
  do begin
      sum:=sum-a[k]*zk;
      zk:=z*zk;
     end;
  norm:=sum;
  sum:=0;
  zk:=1;
  for k:=0 to n
  do begin
      sum:=sum+b[k]*zk;
      zk:=z*zk;
     end;
  norm:=norm/sum;
  for k:=0 to n
  do b[k]:=b[k]*norm;

end;

procedure LP2HP(n:integer);
var k:integer;
begin
for k:=1 to n
do begin
   P[k].r:=-P[k].r;
   P[k].i:=-P[k].i;
   Z[k].r:=-Z[k].r;
   Z[k].i:=-Z[k].i;
   end;
end;


procedure UpdateButton(HWindow:HWnd;FiltName:integer);
begin
  SendDlgItemMessage(HWindow, 107, bm_SetCheck, 0, 0);
  SendDlgItemMessage(HWindow, 108, bm_SetCheck, 0, 0);
  SendDlgItemMessage(HWindow, 109, bm_SetCheck, 0, 0);
  case FiltName of
  zButter:begin
        SendDlgItemMessage(HWindow, 107, bm_SetCheck, 1, 0);
      end;
  zCheb:begin
        SendDlgItemMessage(HWindow, 108, bm_SetCheck, 1, 0);
      end;
  zInvCheb:begin
        SendDlgItemMessage(HWindow, 109, bm_SetCheck, 1, 0);
      end;
  end; (* of case *)
end;

procedure Butter(filterp:pBuiltIn;n:integer;ft:real;filtype:integer);
var analog_w:real;
begin
       filterp^.order:=n;
       filterp^.ft:=ft;
       filterp^.filtype:=filtype;
       if filtype=zHighPass
       then ft:=0.5-ft;
  (* freq warp...*)
       analog_w:=2*sin(Pi*ft)/cos(Pi*ft);
       CalcolaPoli(analog_w,analog_w,n);
       if filtype=zHighPass
       then LP2HP(n);
       Coeff(filterp^.pa^,filterp^.pb^,n);
       case filtype of
         zLowPass:AdjustGain(1,filterp^.pa^,filterp^.pb^,n);
         zHighPass:AdjustGain(-1,filterp^.pa^,filterp^.pb^,n);
         end; (* of case *)
end;

procedure Cheb(filterp:pBuiltIn;n:integer;ft,ripple:real;filtype:integer);

(* ripple e' espresso in (db) *)
(* ft e' una frequenza normalizzata, range: 0 .. 0.5 *)

var analog_w,ex:real;
begin
       filterp^.order:=n;
       filterp^.ft:=ft;
       filterp^.ripple:=ripple;
       filterp^.filtype:=filtype;
       if filtype=zHighPass
       then ft:=0.5-ft;
  (* freq warp...*)
       analog_w:=2*sin(Pi*ft)/cos(Pi*ft);
       ex:=exp((ripple/10)*ln(10))-1;
       ex:=exp(ln(sqrt(1+1/ex)+1/sqrt(ex))/n);

       CalcolaPoli(analog_w*(ex-1/ex)/2,analog_w*(ex+1/ex)/2,n);
       if filtype=zHighPass
       then LP2HP(n);
       Coeff(filterp^.pa^,filterp^.pb^,n);
       case filtype of
         zLowPass:AdjustGain(1,filterp^.pa^,filterp^.pb^,n);
         zHighPass:AdjustGain(-1,filterp^.pa^,filterp^.pb^,n);
         end; (* of case *)
end;

procedure CalcolaInvPoli(w,aa,bb:real;n:integer);
var al,mm:real;
    i,nz:integer;
begin
(* calcola i poli sulla circonferenza *)
for I:=1 to n
do begin
   AL:=(I/n-1/2/n)*PI;
   P[I].r:=-aa*sin(AL);
   P[I].i:=bb*cos(AL);
   mm:=sqr(p[i].r)+sqr(p[i].i);
   p[i].r:=w * p[i].r / mm;
   p[i].i:=w * p[i].i / mm;
   end;
(* determina il numero di zeri da inserire *)
   if odd(n)
   then nz:=n-1
   else nz:=n;
   for i:=1 to nz div 2
   do begin
      z[i].r:=0;
      z[i].i:=w/cos((i-0.5)*Pi/n);
      end;
   for i:=nz div 2+1 to nz
   do begin
      z[i].r:=0;
      z[i].i:=-z[i-nz div 2].i;
      end;
(* applica la trasformazione bilineare *)
for i:=1 to n
do S2Z(P[i]);
for i:=1 to nz
do S2Z(Z[i]);
(* dopo la trasformazione bilineare ci sono <n-nz> zeri in -1 ! *)
if n<>nz
then begin
      Z[n].r:=-1;Z[n].i:=0;
     end;
end;



procedure InvCheb(filterp:pBuiltIn;n:integer;ft,ripple:real;filtype:integer);

(* ripple e' espresso in (db) *)
(* ft e' una frequenza normalizzata, range: 0 .. 0.5 *)

var analog_w,ex:real;
begin
       filterp^.order:=n;
       filterp^.ft:=ft;
       filterp^.ripple:=ripple;
       filterp^.filtype:=filtype;
       if filtype=zHighPass
       then ft:=0.5-ft;
  (* freq warp...*)
       analog_w:=2*sin(Pi*ft)/cos(Pi*ft);
       ex:=exp((-ripple/10)*ln(10));
       ex:=exp(ln(sqrt(1+1/ex)+1/sqrt(ex))/n);

       CalcolaInvPoli(analog_w,(ex-1/ex)/2,(ex+1/ex)/2,n);
       if filtype=zHighPass
       then LP2HP(n);
       Coeff(filterp^.pa^,filterp^.pb^,n);
       case filtype of
         zLowPass:AdjustGain(1,filterp^.pa^,filterp^.pb^,n);
         zHighPass:AdjustGain(-1,filterp^.pa^,filterp^.pb^,n);
         end; (* of case *)
end;

(*---- generic  ----*)

procedure SetUpFilterDialog(HWindow:HWND;id:integer;filterp:pBuiltIn);
var
  testo:array[0..31]of char;
begin
  SendDlgItemMessage(HWindow, id, cb_InsertString, zLowPass, longint(StrCopy(testo,'LowPass')));
  SendDlgItemMessage(HWindow, id, cb_InsertString, zHighPass, longint(StrCopy(testo,'HighPass')));
  if filterp <> nil
  then with filterp^
       do begin
          SendDlgItemMessage(HWindow, id, cb_SetCurSel, filtype, 0);
          Str(ft:10:5,testo);
          SetDlgItemText(HWindow, 104, testo);
          SetDlgItemInt(HWindow,105,order,false);
          case filtname of
            zCheb,zInvCheb:begin
                   Str(ripple:10:5,testo);
                   SetDlgItemText(HWindow, 106, testo);
                  end;
            end;     
          UpdateButton(HWindow,filtname);
          end
  else begin
(*
  Il filtro di default e' zButter
*)
       SendDlgItemMessage(HWindow, id, cb_SetCurSel, zLowPass, 0);
       SetDlgItemText(HWindow, 104, StrCopy(testo,'0.0'));
       SetDlgItemInt(HWindow,105,0,false);
       UpdateButton(HWindow,zButter);
       end
end;

procedure FilterDialogOk(HWindow:HWND;id:integer;var newp:pBuiltIn);
var testo:array[0..63]of char;
    i:longint;
     ft,ripple:real;
     _order,filtname:integer;
     selindex,code:integer;
     result:WordBool;

begin
  newp:=nil;
  GetDlgItemText(HWindow, 104, testo, sizeof(testo));
  if strlen(testo) = 0
  then code:=-1
  else Val(testo,ft,code);
  if (code=0)and(ft > 0)and(ft < 0.5)
  then begin
       _order:=GetDlgItemInt(HWindow, 105, pBool(@result), false);
       if result and (_order > 0) and (_order <= 1024)
       then begin
            if SendDlgItemMessage(HWindow, 107, bm_GetCheck, 0, 0) = 1
            then filtname:=zButter
            else if SendDlgItemMessage(HWindow, 108, bm_GetCheck, 0, 0) = 1
                 then filtname:=zCheb
                 else filtname:=zInvCheb;
            SelIndex:=SendDlgItemMessage(HWindow, id, cb_GetCurSel, 0, 0);
            if (filtname=zCheb) or (filtname=zInvCheb)
            then begin
                 GetDlgItemText(HWindow, 106, testo, sizeof(testo));
                 if strlen(testo) = 0
                 then code:=-1
                 else Val(testo,ripple,code);
                 if (code<>0)or(ripple<=0)
                 then StrCopy(testo,'<ripple> must be a positive real')
                 else if NewBuiltIn(newp,_order,ft,ripple,SelIndex,filtname) = 0
                      then exit
                      else StrCopy(testo,'<order> is not in allowed range');
                 end
            else begin
                 if NewBuiltIn(newp,_order,ft,0,SelIndex,filtname) = 0
                 then exit
                 else StrCopy(testo,'<order> is not in allowed range');
                 end;
            end
       else StrCopy(testo,'<order> must be in range [1..1024]');
       end
  else StrCopy(testo,'<ft> must be a positive real less than 0.5');
  MessageBeep(MB_ICONEXCLAMATION);
  MessageBox(HWindow,testo,'Filter',MB_ICONEXCLAMATION or MB_OK);

end;

function StoreFilterParam(PS:pSystem;var filterp:pBuiltIn;HWindow:HWnd;var st:string;ErrString:PChar):boolean;
var sub:string;j:integer;
    code:integer;
    ft,ripple:real;
    order,filtype,filtname:integer;
    newp:pBuiltIn;
const BadFormat='Wrong format';
begin
  StoreFilterParam:=false;StrCopy(ErrString,BadFormat);
  j:=1;
  if not GetIValue(PS,st,j,filtype)then exit;inc(j);
  if not GetIValue(PS,st,j,filtname)then exit;inc(j);
  if not GetIValue(PS,st,j,order)then exit;inc(j);
  if not GetRValue(PS,st,j,ft)then exit;inc(j);
  if filtname in[zCheb,zInvCheb]
  then begin
       if not GetRValue(PS,st,j,ripple)then exit;inc(j);
       end
  else ripple:=0;
  if (ft <= 0)or (ft >= 0.5)
  then begin
       StrCopy(ErrString,'<ft> must be a positive real less than 0.5');
       exit;
       end;
  if (order <= 0) or (order > MaxOrder)
  then begin
       StrCopy(ErrString,'<order> must be in range [1..1024]');
       exit;
       end;
  if ((filtname=zCheb) or (filtname=zInvCheb)) and (ripple<=0)
  then begin
       StrCopy(ErrString,'<ripple> must be a positive real');
       exit;
       end;
  if NewBuiltIn(newp,order,ft,ripple,filtype,filtname) <> 0
  then begin
       StrCopy(ErrString,'<order> is not in allowed range');
       exit;
       end;
  if filterp <> nil
  then Dispose(filterp,Done);
  filterp:=newp;
  StoreFilterParam:=true;
end;

procedure GetFilterDlg(HWindow:HWND;id:integer;var st:string);
var testo:array[0..63]of char;
     _order,filtname:integer;
begin
  Str(SendDlgItemMessage(HWindow, id, cb_GetCurSel, 0, 0),st);
  if SendDlgItemMessage(HWindow, 107, bm_GetCheck, 0, 0) = 1
  then filtname:=zButter
  else if SendDlgItemMessage(HWindow, 108, bm_GetCheck, 0, 0) = 1
       then filtname:=zCheb
       else filtname:=zInvCheb;
  Str(filtname,testo);
  st:=st+';'+StrPas(testo);
  GetDlgItemText(HWindow, 105, testo, sizeof(testo));
  st:=st+';'+StrPas(testo);
  GetDlgItemText(HWindow, 104, testo, sizeof(testo));
  st:=st+';'+StrPas(testo);
  if (filtname=zCheb) or (filtname=zInvCheb)
  then begin
       GetDlgItemText(HWindow, 106, testo, sizeof(testo));
       st:=st+';'+StrPas(testo);
       end;

end;


function NewBuiltIn(var filterp:pBuiltIn;n:integer;fc,ripple:real;_filtype,_filtname:integer):integer;
(* return code = 0 : Filter has been created
                 1 : <n> is out of range
*)
begin
  if (n <= MaxOrder)and(n>0)
  then begin
       New(filterp,Init(n,n));
       case _filtname of
         zButter:Butter(filterp,n,fc,_filtype);
         zCheb:Cheb(filterp,n,fc,ripple,_filtype);
         zInvCheb:InvCheb(filterp,n,fc,ripple,_filtype);
         end;
       filterp^.filtname:=_filtname;
       NewBuiltIn:=0;
       end
  else NewBuiltIn:=1;
end;

end.