unit WSMCI;

(* QUESTA UNITA' GESTISCE UN DIALOGO PER LA
   RIPRODUZIONE E REGISTRAZIONE DI UN FILE *.WAV.
   SONO DISPONIBILI I COMANDI <PAUSE/RESUME> <STOP>.
   RICHIEDE WINDOWS 3.1 E UNA PERIFERICA MCI (SB/MSS...)

   UTILIZZA LA PROCEDURA   'mciSendString'

*)
interface

uses Strings, WinTypes, WinProcs, WObjects, Win31,
  MMSystem;

{$R MCIWAVE}

function PlayWave(HWindow:HWND; WaveName:PChar):integer;
function RecordWave(HWindow:HWND; WaveName:PChar; fsample:longint):integer;

implementation

const
  id_Current  = 100;
  id_Total    = 101;
{ Menu command IDs }

  id_Pause = 102;
  id_Stop  = 103;
  Timer_Id  = 999;  { Unique timer ID. }

type

{ Filename string }

  TFilename = array[0..255] of Char;

{ Parameter record }
  PRecordParam = ^TRecordParam;
  TRecordParam = record
                   fs:longint;
                   fname:PChar;
                 end;
  
{ Application main window }

  PPlayDialog = ^TPlayDialog;
  TPlayDialog = object
    ElementName: TFilename;
    TimerGoing  : Boolean;
    IsRunning  : Boolean;
    Paused     : Boolean;

    destructor  Done; virtual;

    procedure StopWave;          virtual;

    function CMSoundPlay(HWindow:HWND):integer;
    procedure Pause(HWindow:HWND);
    constructor Init(ATitle: PChar; WaveName:PChar);
  end;

  PRecordDialog = ^TRecordDialog;
  TRecordDialog = object
    ElementName: TFilename;
    fsample : longint;
    TimerGoing  : Boolean;
    IsRunning  : Boolean;
    Paused     : Boolean;

    destructor  Done; virtual;

    procedure StopWave;          virtual;

    function CMSoundPlay(HWindow:HWND):integer;
    procedure Pause(HWindow:HWND);
    constructor Init(ATitle: PChar; WaveName:PChar; fs: longint);
  end;
(* Esempio di uso del dialogo TPlayDialog

  Res:=PlayWave(HWindow,'c:\tpw\chord.wav')));
  if Res <> 0
  then begin
         mciGetErrorString(Res, ErrMsg, SizeOf(ErrMsg));
         MessageBox(HWindow, ErrMsg, 'Play Wave', mb_OK);
       end;

*)


var Objectp:PPlayDialog;

function PlayDlg(HWindow: HWnd; Message: Word; wParam: Word;
  lParam: Longint): Longint; export;
var
  testo:array[0..31]of char;
  s:string[31];
  res:integer;
begin
  case Message of
    wm_InitDialog:
      begin
       Objectp:=new(PPlayDialog,Init('Play',pChar(LParam)));
       res:=Objectp^.CMSoundPlay(HWindow);
       if res <> 0
       then EndDialog(HWindow, res);
      end;
    wm_Command:
      case wParam of
	idCancel:
	  EndDialog(HWindow, 0);
        id_Pause:Objectp^.Pause(HWindow);
        id_Stop:EndDialog(HWindow, 0);
      else
	PlayDlg := 0;
      end;
    wm_Destroy:begin
               if Objectp^.TimerGoing
               then KillTimer(HWindow, Timer_Id);
               Objectp^.Done;
               end;
    mm_MCINotify:EndDialog(HWindow,0);
    wm_Timer:begin
                 mciSendString ('status mywave position', testo, 30, 0);
                 SetDlgItemText(HWindow,id_Current,testo);
             end;
  else
    PlayDlg := 0;
  end;
end;

var RecordP:PRecordDialog;

function RecordDlg(HWindow: HWnd; Message: Word; wParam: Word;
  lParam: Longint): Longint; export;
var
  testo:array[0..31]of char;
  s:string[31];
  res:integer;
begin
  case Message of
    wm_InitDialog:
      begin
       RecordP:=new(PRecordDialog,Init('Record',PRecordParam(LParam)^.fname,PRecordParam(LParam)^.fs));
       res:=RecordP^.CMSoundPlay(HWindow);
       if res <> 0
       then EndDialog(HWindow, res);
      end;
    wm_Command:
      case wParam of
	idCancel:
	  EndDialog(HWindow, 0);
        id_Pause:RecordP^.Pause(HWindow);
        id_Stop:EndDialog(HWindow, 0);
      else
	RecordDlg := 0;
      end;
    wm_Destroy:begin
               if RecordP^.TimerGoing
               then KillTimer(HWindow, Timer_Id);
               RecordP^.Done;
               end;
    mm_MCINotify:EndDialog(HWindow,0);
    wm_Timer:begin
                 mciSendString ('status mywave position', testo, 30, 0);
                 SetDlgItemText(HWindow,id_Current,testo);
             end;
  else
    RecordDlg := 0;
  end;
end;



{ Plays the wave on request.
}
function TPlayDialog.CMSoundPlay(HWindow:HWND):integer;
var
  Res    : Longint;
  Command: array [0..127] of Char;
  buffer:array[0..31]of char;
  pVar:pointer;
begin
  CMSoundPlay:=0;
  if not IsRunning then
  begin
{ MCI APIs to open a device and play a .WAV file, using notification to close
}
    pVar:=@ElementName;
    WVSprintf(Command,'open %s type waveaudio alias mywave',pVar);
    Res:=mciSendString (Command, Buffer, 30, 0);
    if Res <> 0
    then begin
           CMSoundPlay:=Res;
           exit;
         end
    else begin
         mciSendString ('set mywave time format ms', nil, 0, 0);
         mciSendString ('status mywave length', buffer, 30, 0);
         SetDlgItemText(HWindow,id_Total,buffer);
         Res:=mciSendString ('play mywave notify', Buffer, 30, HWindow);
         if Res <> 0
         then begin
              mciSendString ('close mywave', nil, 0, 0);
              CMSoundPlay:=Res;
              exit;
              end;

{ Make sure the Play/Stop toggle menu knows we're running.
}
        IsRunning := True; 
        TimerGoing := (SetTimer(HWindow, Timer_Id, 500, nil) <> 0);

        end;
      end
  else begin
         StopWave;
       end;
end;

{ Pauses or resumes the playback in response to requests to do so from
  the menu.  The File | Pause selection acts as a toggle.
}
procedure TPlayDialog.Pause(HWindow:HWND);
begin
  if not Paused then
  begin       { Pause the playing. }
    mciSendString ('pause mywave', nil, 0, 0);
    SetDlgItemText(HWindow,id_pause,'&Resume');
  end
  else
  begin       { Resume the playing. }
    mciSendString ('resume mywave', nil, 0, 0);
    SetDlgItemText(HWindow,id_pause,'&Pause');

  end;

  Paused := not Paused;
end;

{ Stops the playing waveform file, and closes the waveform device.
}
procedure TPlayDialog.StopWave;
begin
  if IsRunning then
  begin
    (*Res :=*) mciSendString ('stop mywave', nil, 0, 0);
    (*Res :=*) mciSendString ('close mywave', nil, 0, 0);

    IsRunning := FALSE;
  end;
end;


constructor TPlayDialog.Init(ATitle: PChar; WaveName:PChar);
begin
  StrCopy(ElementName,WaveName);
  IsRunning  := False;
  Paused     := False;
  TimerGoing := False;
end;

destructor TPlayDialog.Done;
begin
  StopWave;
end;

function PlayWave(HWindow:HWND; WaveName:PChar):integer;
begin
  PlayWave:=DialogBoxParam(HInstance, 'Play', HWindow, @PlayDlg, longint(WaveName));
end;

{ Records the wave on request.
}
function TRecordDialog.CMSoundPlay(HWindow:HWND):integer;
var
  Res    : Longint;
  Command: array [0..127] of Char;
  buffer:array[0..31]of char;
  Arg: array[0..1]of longint;
begin
  CMSoundPlay:=0;
  if not IsRunning then
  begin
{ MCI APIs to open a device and play a .WAV file, using notification to close
}
    StrCopy(Command,'open new type waveaudio alias mywave');
    Res:=mciSendString (Command, Buffer, 30, 0);
    if Res <> 0
    then begin
           CMSoundPlay:=Res;
           exit;
         end
    else begin
         mciSendString ('set mywave time format ms', nil, 0, 0);
         Arg[0]:=fsample;
         Arg[1]:=fsample;
         WVSprintf(Command,'set mywave samplespersec %lu bytespersec %lu bitspersample 8',Arg);
         Res:=mciSendString (Command, Buffer, 30, 0);
         if Res <> 0
         then begin
              mciSendString ('close mywave', nil, 0, 0);
              CMSoundPlay:=Res;
              exit;
              end;
         Res:=mciSendString ('record mywave notify', Buffer, 30, HWindow);
         if Res <> 0
         then begin
              mciSendString ('close mywave', nil, 0, 0);
              CMSoundPlay:=Res;
              exit;
              end;

{ Make sure the Record/Stop toggle menu knows we're running.
}
        IsRunning := True; 
        TimerGoing := (SetTimer(HWindow, Timer_Id, 500, nil) <> 0);

        end;
      end
  else begin
         StopWave;
       end;
end;

{ Pauses or resumes the playback in response to requests to do so from
  the menu.  The File | Pause selection acts as a toggle.
}
procedure TRecordDialog.Pause(HWindow:HWND);
begin
  if not Paused then
  begin       { Pause the recording. }
    mciSendString ('pause mywave', nil, 0, 0);
    SetDlgItemText(HWindow,id_pause,'&Resume');
  end
  else
  begin       { Resume the recording. }
    mciSendString ('resume mywave', nil, 0, 0);
    SetDlgItemText(HWindow,id_pause,'&Pause');

  end;

  Paused := not Paused;
end;

{ Stops the recording waveform file, and closes the waveform device.
}
procedure TRecordDialog.StopWave;
var  pVar:pointer;
     Command: array [0..127] of Char;
     buffer:array[0..31]of char;
begin
  if IsRunning then
  begin
    (*Res :=*) mciSendString ('stop mywave', nil, 0, 0);
    pVar:=@ElementName;
    WVSprintf(Command,'save mywave %s',pVar);
    (*Res:=*)mciSendString (Command, Buffer, 30, 0);
    (*Res :=*) mciSendString ('close mywave', nil, 0, 0);

    IsRunning := FALSE;
  end;
end;


constructor TRecordDialog.Init(ATitle: PChar; WaveName:PChar; fs: longint);
begin
  StrCopy(ElementName,WaveName);
  fsample    := fs;
  IsRunning  := False;
  Paused     := False;
  TimerGoing := False;
end;

destructor TRecordDialog.Done;
begin
  StopWave;
end;

function RecordWave(HWindow:HWND; WaveName:PChar; fsample:longint):integer;
var Param:TRecordParam;
begin
  Param.fs:=fsample;
  Param.fname:=WaveName;
  RecordWave:=DialogBoxParam(HInstance, 'Record_Dlg', HWindow, @RecordDlg, longint(@Param));
end;

end.